/*
** Module   : DojoToolkitHandler.java
** Abstract : delivers Dojo toolkit JS modules.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140109 First version.
*/

package com.goldencode.p2j.ui.client.chui.driver.web;

import java.io.*;
import java.util.logging.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.eclipse.jetty.server.handler.*;
import org.eclipse.jetty.server.*;

import com.goldencode.p2j.util.*;

/**
 * Jetty Handler which delivers Dojo toolkit modules.
 * The Dojo toolkit distribution zip file should be located in the classpath
 * and modules are accessed as resources.  
 * The Dojo module target request should start with DOJO_TOOLKIT_ROOT.
 * Starting from target the Dojo module resource name is constructed 
 * by replacing the DOJO_TOOLKIT_ROOT with DOJO_TOOLKIT.
 * Than the module resource is loaded and sent as response to requester.
 */
public class DojoToolkitHandler
extends AbstractHandler
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(DojoToolkitHandler.class.getName());

   /** Dojo toolkit package */
   private static final String DOJO_TOOLKIT = "dojo-release-1.9.2";

   /** Dojo toolkit root path */
   private static final String DOJO_TOOLKIT_ROOT = "/dojo-toolkit";
   
   /**
    * Return a Dojo JS file (module) if found or empty response and 
    * HttpServletResponse.SC_NOT_FOUND if module was not found.
    *    
    * @param   target 
    *          The target of the request - either a URI or a name.
    * @param   base
    *          The base request.
    * @param   request
    *          The object or a wrapper of the request.
    * @param   response 
    *          The object or a wrapper of the response. 
    */
   public void handle(String              target, 
                      Request             base,
                      HttpServletRequest  request, 
                      HttpServletResponse response)
   throws IOException, 
          ServletException
   {
      // Check if Dojo module request
      boolean isDojoModule = target.toLowerCase().startsWith(DOJO_TOOLKIT_ROOT);
      // If Dojo module request try to deliver the module. 
      if (!base.isHandled() && isDojoModule)
      {
         deliverDojoModule(target, base, response);
      }    
   }
      
   /**
    * Deliver a Dojo toolkit module.
    *
    * @param   module
    *          Dojo module path and name.
    * @param   base
    *          The details and state the request.
    * @param   response 
    *          The details and state of the response.
    *           
    * @throws  IOException
    *          ServletException
    *          If any error conditions occur.
    */
   private void deliverDojoModule(String module, Request base, HttpServletResponse response)
   throws IOException, 
          ServletException
   {
      String dojoModule = module.replace(DOJO_TOOLKIT_ROOT, DOJO_TOOLKIT);
      
      LOG.logp(Level.INFO,
               "DojoToolkitHandler#deliverDojoModule()",
               "",
               LogHelper.generate("Delivering %s module.", dojoModule));

      // content type
      response.setContentType("application/javascript");
      
      PrintWriter writer = response.getWriter();
      
      InputStream is = Thread.currentThread().getContextClassLoader().
                                              getResourceAsStream(dojoModule);
      if (is != null)
      {
         response.setStatus(HttpServletResponse.SC_OK);
         BufferedReader reader = new BufferedReader(new InputStreamReader(is));
      
         String str = null;     
         while (null != (str = reader.readLine()))
         {
            writer.println(str);
         }      
               
         reader.close();
      }
      else
      {
         // module not found
         LOG.logp(Level.WARNING,
                  "DojoToolkitHandler#deliverDojoModule()",
                  "",
                  LogHelper.generate("Module %s not found!", dojoModule));
         response.setStatus(HttpServletResponse.SC_NOT_FOUND);         
      }

      // mark the request as handled
      base.setHandled(true);
   }
}

